@@ -1,7 +1,7 @@
 from concurrent.futures import ThreadPoolExecutor
 from time import sleep
-from collections.abc import Sequence 
-from threading import Thread, Lock
+from collections.abc import Sequence
+import random
 import queue
 import threading
 
@@ -180,6 +180,22 @@ class FilterIterator:
             return self.__next__()
 
 
+class FileChunkIterator:
+    def __init__(self, filename, buffer):
+        self.filename = filename
+        self.fd = open(filename, "rb")
+        self.buffer = buffer
+
+    def __next__(self):
+        count = self.fd.readinto(self.buffer)
+        if count is None or count == 0:
+            raise StopIteration
+        return self.buffer[:count]
+
+    def close(self):
+        self.fd.close()
+
+
 class FileLineIterator:
     def __init__(self, filename):
         self.filename = filename
@@ -206,6 +222,53 @@ class BlockingQueueIterator:
         else:
             return next
 
+
+class IterativeIterator:
+    def __init__(self, seed, func):
+        self.seed = seed
+        self.func = func
+
+    def __next__(self):
+        result = self.seed
+        self.seed = self.func(self.seed)
+        return result
+
+
+class RandomIntIterator:
+    def __init__(self, low, high):
+        self.low = low
+        self.high = high
+
+    def __next__(self):
+        return random.randrange(self.low, self.high)
+
+
+class RandomFloatIterator:
+    def __init__(self, scale=1):
+        self.scale = scale
+
+    def __next__(self):
+        return random.random() * self.scale
+
+
+class RandomCharIterator:
+    def __init__(self, seed):
+        self.seed = seed
+        
+    def __next__(self):
+        return self.seed[random.randrange(0, len(self.seed))]
+
+
+class RandomStringIterator:
+    def __init__(self, seed, length):
+        self.seed = seed
+        self.length = length
+        
+    def __next__(self):
+        chars = [self.seed[random.randrange(0, len(self.seed))] for _ in range(0, self.length)]
+        return "".join(chars)
+
+
 class EndawareIterator:
     def __init__(self, src, queue, last):
         self.src = src
@@ -223,7 +286,15 @@ class EndawareIterator:
                 q.put(self.last)
             raise StopIteration
 
+
 class Stream:
+    digits = "0123456789"
+    alphabets_lower = "abcdefghijklmnopqrstuvwxyz"
+    alphabets_upper = alphabets_lower.upper()
+    default_chars = digits + alphabets_lower + alphabets_lower
+    hex_chars_upper = "0123456789ABCDEF"
+    hex_chars_lower = hex_chars_upper.lower()
+    
     def __init__(self, src, begin_func=None, exit_func=None):
         """Create a stream from an iterator (e.g. iter(list)), or an iterable (e.g. list)
         Optionally, pass in a begin func (when used in WITH keyword), and exit_func (for cleanup)
@@ -333,10 +404,64 @@ class Stream:
         result = Stream(file_iter, None, lambda: file_iter.close())
         return result
 
+    @staticmethod
+    def from_file_chunks(filename, buffer=None, buffer_size=4096):
+        """Read file in binary, by chunks. Each stream element is a byte array
+        Stream exhausts when file is fully read
+        If buffer (bytearray) is specified, buffer is used.
+        Otherwise, if buffer_size is specified, new bytearray(buffer_size) would be used.
+        buffer_size, when not specified, defaults to 4096 bytes.
+
+        This should be faster than allocating new buffer on every new read.
+
+        This iterator re-uses buffer, so it should be much faster with file.read!
+        """
+        if buffer is None:
+            buffer = bytearray(buffer_size)
+        file_iter = FileChunkIterator(filename, buffer)
+        return Stream(file_iter, None, file_iter.close)
+
     @staticmethod
     def generate(func):
         return Stream(GeneratorIterator(func))
 
+    @staticmethod
+    def iterate(seed, func):
+        """Create stream who's first value is seed. subsequent values are repeated application of seed to the func."""
+        return Stream(IterativeIterator(seed, func))
+
+    @staticmethod
+    def random_number_strings(length=5):
+        return Stream.random_strings(Stream.digits, length)
+    
+    @staticmethod
+    def random_alphabets(length=5, lower=False):
+        if lower:
+            return Stream.random_strings(Stream.alphabets_lower, length)
+        return Stream.random_strings(Stream.alphabets_upper, length)
+    
+    @staticmethod
+    def random_hex_strings(length=5, lower=False):
+        if lower:
+            return Stream.random_strings(Stream.hex_chars_lower, length)
+        return Stream.random_strings(Stream.hex_chars_upper, length)
+
+    @staticmethod
+    def random_strings(chars=default_chars, length=5):
+        return Stream(RandomStringIterator(chars, length))
+
+    @staticmethod
+    def random_chars(chars=default_chars):
+        return Stream(RandomCharIterator(chars))
+
+    @staticmethod
+    def random_ints(low, high):
+        return Stream(RandomIntIterator(low, high))
+
+    @staticmethod
+    def random_floats(scale=1):
+        return Stream(RandomFloatIterator(scale))
+
     def min(self):
         """Similar to max, but find minimum. This consumes the stream"""
         def min_cmp(a, b):
@@ -411,15 +536,16 @@ class Stream:
     def uniq(self):
         """Create stream of unique elements. Order is preserved.
         Note that this consumes all elements! Don't use on unbounded stream or super large streams."""
-        consumed = list(self)
         emitted = set()
+
         def emit_filter(x):
             if x in emitted:
                 return False
             else:
                 emitted.add(x)
                 return True
-        return Stream(consumed, self.begin_func, self.exit_func).filter(emit_filter)
+
+        return self.filter(emit_filter)
 
     def to_list(self):
         """Convert stream to list. alternatively, list(stream) does the samething, since stream itself is iterable.
@@ -472,7 +598,7 @@ class Stream:
         return Stream(ConcatIterator(self.src, next.src), new_begin, new_exit)
 
     def split(self, count=2):
-        """Split this stream into 2 streams. First one is primary stream, second one consumption is depend on consumption by first one.
+        """Split this stream into count streams. First one is primary stream, subsequent ones' consumption is depend on consumption by first one.
         The elements are available on second one only after first one had consumed it.
         Remember to consume first split stream.
 
@@ -597,28 +723,47 @@ if __name__ == "__main__":
     s1, s2, s3, s4 = Stream([1,2,3,4,5],lambda:print("Begin"), lambda:print("end")).split(4)
     
     def slow_consume(name, stream):
-        iter = stream.__iter__()
-        while True:
-            try:
-                i = iter.__next__()
-                print(f"{name} => consumed {i}")
-            except StopIteration:
-                break
+        for i in stream:
+            print(f"{name} => consumed {i}")
             sleep(1)
 
     def consume_asap(name, stream):
         for i in stream:
             print(f"{name} => consumed {i}")
-            
-    with s1:
-      with s2:
-        with s3:
-          with s4:
-            t1 = threading.Thread(target=slow_consume, args=("s1", s1))
-            t2 = threading.Thread(target=consume_asap, args=("s2", s2))
-            t3 = threading.Thread(target=consume_asap, args=("s3", s3))
-            t4 = threading.Thread(target=consume_asap, args=("s4", s4))
-            Stream([t1, t2, t3, t4]).for_each(lambda x: x.start())
-            Stream([t1, t2, t3, t4]).for_each(lambda x: x.join())
 
-    # if you run s2.for_each, it will block forever!
\ No newline at end of file
+    with s1, s2, s3, s4:
+        t1 = threading.Thread(target=slow_consume, args=("s1", s1))
+        t2 = threading.Thread(target=consume_asap, args=("s2", s2))
+        t3 = threading.Thread(target=consume_asap, args=("s3", s3))
+        t4 = threading.Thread(target=consume_asap, args=("s4", s4))
+        Stream([t1, t2, t3, t4]).for_each(lambda x: x.start())
+        Stream([t1, t2, t3, t4]).for_each(lambda x: x.join())
+
+    # if you run s2.for_each, it will block forever!
+    def adder(x):
+        return x + 1
+    
+    Stream.iterate(1, adder).limit(10).for_each(print)
+
+    # Copy file
+    with Stream.from_file_chunks("src/pystream/pystream.py").skip(0) as stream:
+        with open("output.py", "wb") as output:
+            stream.for_each(lambda x: output.write(x))
+
+    Stream.random_strings("ABCDEFG", 12).limit(20).for_each(print)
+    Stream.random_chars("ABCDEFG").limit(20).for_each(print)
+    Stream.random_ints(0, 10).limit(20).for_each(print)
+    Stream.random_floats(10).limit(20).for_each(print)
+    print("Lower")
+    Stream.random_alphabets(length=5, lower=False).limit(20).for_each(print)
+    print("Lower")
+    Stream.random_alphabets(length=6, lower=True).limit(20).for_each(print)
+    
+    print("Upper ")
+    Stream.random_hex_strings(length=5).limit(20).for_each(print)
+    
+    print("Lower hex")
+    Stream.random_hex_strings(length=10, lower=True).limit(20).for_each(print)
+
+    list_new = [x for x in Stream.random_strings().limit(10)]
+    print(list_new)
\ No newline at end of file
